<?php

App::uses('BoostCakeFormHelper', 'BoostCake.View/Helper');

class FormExtHelper extends BoostCakeFormHelper {

	public $_inputOptions = array(
		'inputDefaults' => array(
				'empty' => '',
				'div' => 'form-group',
				'label' => array(
					'class' => 'col-xs-12 col-sm-2 col-md-3 control-label'
				),
				'class' => 'form-control',
				'wrapInput' => 'col-xs-12 col-sm-10 col-md-9',
		),
		'type' => 'file',
		'class' => 'form-horizontal'
	);

	public function __construct(View $View, $settings = array()) {
		$this->helpers[] = 'Time';

		parent::__construct($View, $settings);
	}

	/**
	 * Radio input
	 *
	 * Used to override the boostcake radio handler as it does not format data to bootstrap standards.
	 * Known issue but as boostcake is not actively developed it look there's no update, so this is
	 * here to work around it.
	 *
	 * @param  string $fieldName    name of the field
	 * @param  array  $radioOptions array of radio options
	 * @param  array  $options      array of input options
	 * @return string               returns the generated html
	 */
	public function radio($fieldName, $radioOptions = array(), $options = array()) {
		$options['legend'] = false;
		$options['separator'] = "\n";
		$afterData = false;

		// Pull the after option if its provided
		if (isset($options['after']) && ! empty($options['after'])) {
			$afterData = $options['after'];
			unset($options['after']);
		}

		// Backup label incase one isn't provided
		$label = Inflector::humanize(Inflector::pluralize(Inflector::underscore($fieldName)));

		// Use the provided label if available
		if (isset($options['label']) && ! empty($options['label'])) {
			$label = $options['label'];
		}

		// Run through cakes (NOT BOOTSTCAKES) radio handler
		$out = parent::radio($fieldName, $radioOptions, $options);

		// Now manipulate it to work with bootstraps default layout as boostcake seeminly doesnt do that...
		$out = explode("\n", $out);
		foreach ($out as $key => &$_out) {
			$input = strip_tags($_out, '<input><img><label>');
			if ($input) {
				$_out = $this->Html->tag('div', $input, array('class' => 'radio'));
			}
		}

		// put it all back together
		$out = implode("\n", $out);

		// wrap in a 9 point column
		$out = $this->Html->tag('div', $out, array('class' => 'col-md-9'));

		// add the label to the output
		$label = $this->Html->tag('label', $label, array('class' => 'col-md-3 control-label'));
		$out = $label . $out;

		// Apply displayInfo if provided
		if ($afterData) {
			$out .= $afterData;
		}

		return $out;
	}

	/**
	 * Returns a textarea marked up for a WYSIWYG editor
	 */
	public function wysiwyg($name, $options) {
		$defaultOptions = array(
			'type' => 'textarea',
			'rows' => 10,
			'cols' => 60
		);

		$options = array_merge($defaultOptions, $options);

		return $this->input($name, $options);
	}

	/**
	 * Creates an Inline Edit form field
	 */
	public function inline($fieldName, $options = array()) {
		$options = $this->_initInputField($fieldName, $options);
		$value = null;

		if (array_key_exists('value', $options)) {
			$value = $options['value'];
			if (!array_key_exists('escape', $options) || $options['escape'] !== false) {
				$value = h($value);
			}
			unset($options['value']);
		}

		return $this->Html->div("inline-edit", $value, array('id' => $options['name'], 'escape' => false));
	}

	/**
	 * Return multicheck area
	 *
	 * @param string $name
	 * @param array $options
	 * @param array $attr
	 * @return string
	 */
	public function multicheck($name, $attr = array()) {
		// try to find the results so we can prepopulate
		$results = Hash::get($this->request->data, $name);
		$selected = array();
		if (isset($results['0']) && is_array($results['0'])) {

			$title = (array_key_exists('title', $results['0'])) ? 'title' : 'name';
			$selected = Hash::combine($results, '{n}.id', '{n}.' . $title);

		} elseif (isset($results) && is_array($results)) {

			$selected = array();
			foreach ($results as $row) {
				$selected[$row] = $row;
			}
		}

		// assign selected fields
		if (!empty($selected)) {
			$selected = array_keys($selected);
		}

		$label = isset($attr['label']) ? $attr['label'] : 'Tagged ' . Inflector::humanize(Inflector::pluralize(Inflector::underscore($name)));

		//if (isset($attr['selectAll']) && $attr['selectAll']) {
			if (! isset($attr['before'])) {
				$attr['before'] = '';
			}

			$attr['before'] .= '<div class="checkbox-option"><a href="#" class="btn btn-sm btn-primary multi-select-all">Select All</a> <a href="#" class="btn btn-sm btn-primary multi-unselect-all">Unselect All</a></div>';
			$this->Html->scriptBlock(
				"$(document).ready(function() {
					$('.multi-select-all').bind('click', function(e) {
						e.preventDefault();

						$(this).parent().parent().find('input[type=checkbox]').prop('checked', true);
					});

					$('.multi-unselect-all').bind('click', function(e) {
						e.preventDefault();

						$(this).parent().parent().find('input[type=checkbox]').prop('checked', false);
					});
				});",
				array('inline' => false)
			);
		//}

		// for multichecks, we move the attr['after'] into its own variable and pass it in separately,
		// or otherwise the HTML gets messed up due to the $this->input()
		$after = '';
		if (!empty($attr['after'])) {
			$after = $attr['after'];
			unset($attr['after']);
		}

		$attr = array_merge($attr,
			array(
				'type' => 'select',
				'multiple' => 'checkbox',
				'div' => 'form-group-multicheck',
				'label' => false,
				'selected' => $selected,
				'options' => (! empty($attr['options'])) ? $attr['options'] : null,
				'disabled' => (! empty($attr['disabled'])) ? $attr['disabled'] : null,
				'class' => 'radio col-xs-12 col-sm-6',
				'wrapInput' => false
			)
		);

		$input = $this->input($name, $attr);

		return $this->_wrapMultcheckboxes(
			$label,
			$input,
			$after
		);
	}

	/**
	 * Helper function for multicheck method
	 * Wraps the supplied input element in the necessary HTML markup
	 */
	protected function _wrapMultcheckboxes($label, $input, $after = '') {
		$markup = '<label class="col-xs-12 col-sm-3 control-label">' . $label . '</label>';
		$markup .= '<div class="col-xs-12 col-sm-9">';

		$markup .= $input;
		if (!empty($after)) {
			$markup .= $after;
		}
		$markup .= '</div>';

		return $markup;
	}

	/**
	 * map fields
	 *
	 * Used to build google maps. Because of boostcake limitations this uses a ton of custom inline
	 * html. If you choose to update/improve, please make sure it follows bootstrap (not cake or inuit)
	 * coding standards!
	 *
	 * @param  string $field   name of the field
	 * @param  array  $options all field options
	 * @return string          html output of the entire map row
	 */
	protected function _map($field, $options = array()) {
		if (! isset($options['label']) || empty($options['label'])) {
			$options['label'] = 'Location';
		}

		$after = '';
		if (! empty($options['after'])) {
			$after = $options['after'];
		}

		$divClass = 'map ';
		$wrapClass = null;
		$labelClass = 'control-label';

		if (isset($this->_inputOptions['div'])) {
			$divClass .= $this->_inputOptions['div'];
		}

		if (isset($this->_inputOptions['wrapInput'])) {
			$wrapClass = $this->_inputOptions['wrapInput'];
		}

		if (isset($this->_inputOptions['label']['class'])) {
			$labelClass = $this->_inputOptions['label']['class'];
		}

		$out = '<div class="' . $divClass . '">';

			$out .= $this->Html->tag('label', $options['label'], array('class' => $labelClass));

			$out .= '<div class="' . $wrapClass . '">';

				$out .= $this->input('google_address', array_merge(
					$options,
					array(
						'label' => false,
						'class' => 'map-address',
						'div' => false,
						'wrapClass' => false,
						'before' => '<button class="find-postcode">Find</button><div class="adminmap"></div>',
						'type' => 'text',
						'after' => false

					)
				));

				$out .= $this->input($field,
					array(
						'type' => 'hidden',
						'class' => 'map_data',
						'div' => false,
						'after' => false
					)
				);

			$out .= '</div>';

			// Add in any after bits!
			if (isset($options['after'])) {
				$out .= $options['after'];
			}

		$out .= '</div>';
		return $out;
	}

	public function addField($field, $attr) {
		if ($this->_isFieldName($field, 'id')) {

			$attr['type'] = 'hidden';

		} elseif ($this->_isFieldName($field, 'slug')) {

			$attr['type'] = 'slug';

		} elseif ($this->_isFieldName($field, 'map') || $this->_isFieldName($field, 'map_data')) {

			$attr['type'] = 'map';

		} elseif ($this->_isFieldName($field, 'google_address') || $this->_isFieldName($field, 'longitude')) {

			return;

		}

		// If display only handle and then return before the $this->input code
		if (isset($attr['displayonly']) && $attr['displayonly'] == 'displayonly') {
			return $this->_displayOnly($field, $attr);
		}

		// assign the elements tooltip text, if required
		if (isset($attr['info'])) {
			$helpText = '<div class="help" title="' . $attr['info'] . '">&nbsp;</div>';

			if (isset($attr['label'])) {
				if (isset($attr['label']['text'])) {
					$attr['label']['text'] = $attr['label']['text'] . $helpText;
				} else {
					$attr['label'] = $attr['label'] . $helpText;
				}
			}

			//$tooltip = array('between' => '<div class="help" title="' . $attr['info'] . '">&nbsp;</div>');
			//$attr = array_merge($attr, $tooltip);

			$this->Html->script(array('jquery.tipsy'), array('inline' => false));
		}

		// assign the elements displayable text
		if (isset($attr['displayInfo'])) {

			if (! isset($attr['after'])) {
				$attr['after'] = '';
			}

			$attr['after'] .= '<p class="help-block"><span>' . $attr['displayInfo'] . '</span></p>';
		}

		if (! isset($attr['class'])) {
			$attr['class'] = null;
		}

		$attr['class'] .= ' form-control';

		switch ($attr['type']) {

			case 'hidden':

				return $this->input($field, array_merge($attr, array('type' => 'hidden')));
				break;

			case 'boolean':
				return $this->input($field, array_merge($attr, array('type' => 'checkbox', 'wrapInput' => 'col-xs-12 col-md-9 col-md-offset-3')));
				break;

			case 'color':
				return $this->input($field, array_merge($attr, array('type' => 'color')));
				break;

			case 'date':

				$out = null;

				$attr = $this->value($attr, $field);
				if (empty($attr['gmt']) || $attr['gmt'] === false) {
					// Dates are stored in the database in GMT so convert the date into the
					// user's timezone (remember to convert back to GMT before saving using
					// `CakeTime::toServer($date, Configure::read('Config.timezone'));`).
					$attr['value'] = !empty($attr['value']) ? $this->Time->format($attr['value'], '%Y-%m-%d') : null;
					// Include a hidden _gmt field to check for when modifying the date before
					// saving.
					$out .= $this->hidden($field . '_gmt', array('value' => true));
				}
				unset($attr['gmt']);

				$out .= $this->input($field, array_merge($attr, array('type' => 'text', 'data-type' => 'datepicker', 'default' => '')));
				return $out;
				break;

			case 'datetime':

				$out = null;

				$attr = $this->value($attr, $field);
				if (empty($attr['gmt']) || $attr['gmt'] === false) {
					// Dates are stored in the database in GMT so convert the date into the
					// user's timezone (remember to convert back to GMT before saving using
					// `CakeTime::toServer($date, Configure::read('Config.timezone'));`).
					$attr['value'] = !empty($attr['value']) ? $this->Time->format($attr['value'], '%Y-%m-%d %H:%M:%S') : null;
					// Include a hidden _gmt field to check for when modifying the date before
					// saving.
					$out .= $this->hidden($field . '_gmt', array('value' => true));
				}
				unset($attr['gmt']);

				$this->Html->script(array('jquery-ui-timepicker-addon'), array('inline' => false));
				$out .= $this->input($field, array_merge($attr, array('type' => 'text', 'data-type' => 'datetimepicker', 'default' => '')));
				return $out;
				break;

			case 'year':
				// assign a default min year value of 25 year in the past, if omitted
				if (empty($attr['minYear'])) {
					$attr['minYear'] = date('Y') - 25;
				}

				// assign a default max year value of 1 year in the future, if omitted
				if (empty($attr['maxYear'])) {
					$attr['maxYear'] = date('Y') + 1;
				}

				$defaultAttr = array(
					'dateFormat' => 'Y',
					'default' => date('Y'),
					'div' => 'input select select--small',
					'empty' => true,
					'type' => 'date',
				);

				// if a value is set then hack the value attribute to a full date
				// string as the FormHelper::_getDateTimeValue method cannot handle
				// a 4 digit year string
				$yearValue = Hash::get($this->request->data, $field);
				if (! empty($yearValue)) {
					$defaultAttr['value'] = Hash::insert($this->request->data, $field, $yearValue . '-01-01');
				}

				return $this->input($field, array_merge($attr, $defaultAttr));
				break;

			case 'email':

				return $this->input($field, array_merge($attr, array('type' => 'email')));
				break;

			case 'password':

				return $this->input($field, array_merge($attr, array('type' => 'password')));
				break;

			case 'integer':
			case 'float':
			case 'decimal':

				unset($attr['length']);

				if (substr($field, -3, 3) == '_id') {

					return $this->input(
						$field,
						array_merge(array('empty' => 'Please select…'), $attr, array('type' => 'select'))
					);

				} else {

					return $this->input($field, array_merge($attr, array('type' => 'number', 'size' => 6)));

				}
				break;

			case 'text':
			case 'html_lite':

				return $this->wysiwyg($field, array_merge($attr, array('class' => 'wysiwyg wysiwyg--lite')));
				break;

			case 'html':
				return $this->wysiwyg($field, array_merge($attr, array('class' => 'wysiwyg wysiwyg--full')));
				break;

			case 'multiselect':

				return $this->input($field, array_merge($attr, array('type' => 'select', 'multiple' => true)));
				break;

			case 'select':

				return $this->input($field, array_merge(array('empty' => 'Please select…'), $attr, array('type' => 'select')));
				break;

			case 'multicheck':
				return $this->multicheck($field, $attr);
				break;

			case 'text_plain':

				if (isset($attr['class']) && $attr['class'] != '') {
					$attr['class'] .= ' text-plain';
				} else {
					$attr['class'] = 'text-plain';
				}

				$attr['type'] = 'textarea';
				return $this->input($field, $attr);
				break;

			case 'slug':

				return $this->_routeSlug($field, $attr);
				break;

			case "route-alias":

				return $this->addField($field . '.slug', array_merge(
					$attr,
					array(
						'value' => (isset($attr['value'])) ? $attr['value'] : $this->value($field),
						'data-type' => ' route-alias',
						'class' => 'form-control route-alias-input input-sm'
					)
				)) . $this->input($field, array(
					'type' => 'hidden',
					'data-type' => 'route-alias'
				));

				break;

			case 'map':

				return $this->_map($field, $attr);
				break;

			case 'radio':
				$options = array();

				if (! empty($attr['options'])) {
					$options = $attr['options'];
					unset($attr['options']);
				}

				return $this->radio($field, $options, array_merge($attr, array('type' => 'radio')));
				break;

			case 'string':
			default:

				return $this->input($field, array_merge($attr, array('type' => 'text')));

		}
	}

	/**
	 * Generates a form field for slug aliases (for use with Routable).
	 *
	 * If using slug prefixing the prefix needs handling by the JS (i.e. do not
	 * remove it here).
	 *
	 * @param string $fieldName
	 * @param array $options
	 * @return string
	 */
	protected function _routeSlug($fieldName, $options = array()) {
		$displayUrl = $this->_getDisplayUrl($options);

		$routeDisplay = '<span class="non-edit base-url">' . $displayUrl . '</span>';
		$routeDisplay .= '<span class="non-edit route">';
		$routeDisplay .= '<span class="slug-display">';

		if (isset($options['value'])) {
			$value = $options['value'];
			unset($options['value']);
		} else {
			$value = $this->value($fieldName);
		}

		if (!empty($options['url_prefix'])) {
			$urlPrefix = $options['url_prefix'];
			$value = str_replace($options['url_prefix'], '', $value);
			unset($options['url_prefix']);
		}
		$value = trim($value, "/");

		$routeDisplay .= $value;

		$routeDisplay .= '</span> <a href="#" class="edit-route btn btn-primary btn-sm"><i class="fa fa-pencil"></i> Edit URL</a>';
		$routeDisplay .= '</span>';
		$routeDisplay .= '<div class="input-edit-slug">';

		$stopEditing = '<a href="#" class="stop-editing btn btn-success"><i class="fa fa-check"></i> Done</a></div>';

		$options['label'] = !empty($options['label']) ? $options['label'] : 'URL';

		return $this->input($fieldName, array_merge(
				array(
					'type' => 'string',
					'between' => $routeDisplay,
					'after' => $stopEditing,
					'class' => 'slug',
					'value' => $value,
					'data-url-prefix' => (isset($urlPrefix)) ? $urlPrefix : false
				),
				$options
			)
		);
	}

	/**
	 * Adds fields with the markup for just html no editable
	 */
	protected function _displayOnly($field, $attr) {
		$output = '';

		// assign the input default to the label
		if (empty($attr['label']) && ! empty($this->_inputOptions['label'])) {
			$attr['label'] = $this->_inputOptions['label'];
		}

		$label = null;
		if (isset($attr['label']) && $attr['type'] !== 'radio') {
			$label = $attr['label'];
			unset($attr['label']);
		}

		// get the model / field name
		if (// checking for just model name in field
			preg_match('/[A-Z]/', substr($field, 0, 1)) === 1 &&
			strpos($field, '.') === false // checking for just model name in field
		) {
			$fieldBits = array(
				'0' => $field
			);
			$fieldPath = $field;
		} elseif (strpos($field, '.') !== false) { // check for a model.field name entry

			$fieldBits = explode('.', $field);
			$fieldPath = $field;
		} else {
			// it's just the field name, get the defaultModel from the form helper
			$fieldBits = array(
				'0' => $this->defaultModel,
				'1' => $field
			);
			$fieldPath = $this->defaultModel . "." . $field;
		}

		$label = $this->_inputLabel($field, $label, $attr);

		if (// check to see if it's a select
			$attr['type'] == 'multiselect' ||
			$attr['type'] == 'select' ||
			$attr['type'] == 'radio' ||
			substr($field, -3, 3) == "_id"
		) {

			// check if it's multiselect
			if ($attr['type'] === 'multiselect') {

				if (!empty($this->request->data[$fieldBits['0']])) {

					$results = array();

					// loop the options
					foreach ($this->request->data[$fieldBits['0']] as $item) {

						// check if our option exists, add to array if so
						if (isset($item['name'])) {
							$results[] = $item['name'];
						} elseif (isset($item['title'])) {
							$results[] = $item['title'];
						}
					}

					$value = String::toList($results);

				} else {
					$value = '-';
				}

			} else {

				// check if options have been passed, if not get them
				if (!isset($attr['options'])) {

					$varName = strtolower((isset($fieldBits['1'])) ? $fieldBits['1'] : $fieldBits['0']);
					$varName = Inflector::variable(
						Inflector::pluralize(preg_replace('/_id$/', '', $varName))
					);
					$varOptions = $this->_View->getVar($varName);
					if (is_array($varOptions)) {
						$attr['options'] = $varOptions;
					}
				}

				// get the id number of the option we are showing
				// $id = (!empty($this->request->data[$fieldBits['0']][$fieldBits['1']])) ? $this->request->data[$fieldBits['0']][$fieldBits['1']] : null;
				$id = Hash::get($this->request->data, $fieldPath);
				// if it doesn't exist show a dash, if it does get the options
				if (!is_null($id)) {

					// check if our option exists, display if so. dash if not
					if (!empty($attr['options'][intval($id)])) {

						$value = $attr['options'][intval($id)];

					} else {
						$value = '-';
					}

				} else {
					$value = '-';
				}
			}

		} else {

			$value = Hash::get($this->request->data, $fieldPath);

			if (!is_null($value)) {

				if (in_array($attr['type'], array('text', 'text_plain'))) {

					$value = nl2br(Hash::get($this->request->data, $fieldPath));

				} elseif ($attr['type'] === 'boolean') {

					$value = Hash::get($this->request->data, $fieldPath);
					$value = ($value == '1') ? 'Yes' : 'No';

				} elseif ($attr['type'] === 'datetime') {

					$value = Hash::get($this->request->data, $fieldPath);
					$value = !empty($value) && $value != '0000-00-00 00:00:00' ? $this->Time->format($value, '%d/%m/%Y %H:%M') : '&nbsp;';

				} else {

					$value = $value = Hash::get($this->request->data, $fieldPath);
				}

			} else {

				$value = '-';
			}
		}

		$class = !empty($attr['div']) && is_string($attr['div']) ? $attr['div'] : null;
		$output .= $this->Html->tag(
			'div',
			null,
			array(
				'class' => $class . ''
			)
		);
			$output .= $this->Html->tag(
			'div',
			null,
			array(
				'class' => 'form-group'
			)
		);
			$output .= $label;
			$output .= $this->Html->tag('div', $value, array('class' => 'col-xs-12 col-sm-10 col-md-8 form-control-readonly'));
			$output .= $this->Html->tag('/div');
		$output .= $this->Html->tag('/div');

		return $output;
	}

	/**
	 * Create - helper method to add in the novalidate option as default to
	 * prevent customer browser inline form validate
	 */
	public function create($model = null, $options = array()) {
		return parent::create($model, array_merge($options, array('novalidate' => true, 'role' => 'form')));
	}

	/**
	 * Generates a display URL for a given prefix/slug along with the current host URL
	 */
	protected function _getDisplayUrl($attr) {
		$displayUrl = 'http://' . $_SERVER['HTTP_HOST'] . '/';
		if (! empty($attr['url_prefix'])) {
			$displayUrl .= $attr['url_prefix'];

			if (substr($attr['url_prefix'], -1) != '/') {
				$displayUrl .= '/';
			}
		}

		return $displayUrl;
	}

	protected function _isFieldName($fieldname, $name) {
		$position = - (strlen($name) + 1);

		return substr($fieldname, $position) == ".$name" || $fieldname == $name;
	}

	/**
	 * override to provide an extra class for multiselects
	 *
	 */
	/**
	 * Generate div options for input
	 *
	 * @param array $options Options list.
	 * @return array
	 */
	protected function _divOptions($options) {
		// We need to grab the 'type' from $options before calling the parent
		// as Cake will strip this out.
		$type = !empty($options['type']) ? $options['type'] : null;
		$options = array_merge(parent::_divOptions($options), $options);
		if (!empty($options['type']) && $options['type'] === 'hidden') {
			return array();
		}
		$div = $this->_extractOption('div', $options, true);
		if (!$div) {
			return array();
		}


		$divOptions = array('class' => 'input');

		if ($type === 'select' && isset($options['multiple']) && $options['multiple']) {
			$type .= ' multiselect';
		}

		$divOptions = $this->addClass($divOptions, $type);
		if (is_string($div)) {
			$divOptions['class'] = $div;
		} elseif (is_array($div)) {
			$divOptions = array_merge($divOptions, $div);
		}
		if ($this->_extractOption('required', $options) !== false &&
			$this->_introspectModel($this->model(), 'validates', $this->field())
		) {
			$divOptions = $this->addClass($divOptions, 'required');
		}
		if (!isset($divOptions['tag'])) {
			$divOptions['tag'] = 'div';
		}

		return $divOptions;
	}
}
